/**
********************************************************************************
\file   trace-printf.c

\brief  Trace function using fprintf to stderr

*******************************************************************************/

/*------------------------------------------------------------------------------
Copyright (c) 2017, B&R Industrial Automation GmbH
Copyright (c) 2013, SYSTEC electronik GmbH
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the copyright holders nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
------------------------------------------------------------------------------*/

//------------------------------------------------------------------------------
// includes
//------------------------------------------------------------------------------
#include <stdio.h>
#include <stdarg.h>
#include <trace/trace.h>
#include <string.h>


//============================================================================//
//            P U B L I C   F U N C T I O N S                                 //
//============================================================================//

static char buf[64];
void print_word (uint32_t word) {
	int i;	
	volatile char  * uart_tx = (char *) 0x82000000;
		
	for (i=7; i >= 0; i--) {
		int disp = ((word >> i*4) & 0xf);
		if (disp <= 9 )
			buf[i] = '0'+ disp;
		else
			buf[i] = 'A' + disp - 10; 
	}
	buf[8] = '\r';
	buf[9] = '\n';

	for (i=7; i>=0; i--) {
		while( ! *uart_tx);	  
		*(uart_tx+4) = buf[i];
		*uart_tx = 2;
	}



	
}

void print_string (char * str) {
	volatile char  * uart_tx = (char *) 0x82000000;
	uint8_t i; 
	uint8_t len = strlen(str); 
	for (i=0; i < len; i++) {
		while (! *uart_tx);
		*(uart_tx+4) = str[i];
		*uart_tx = 2;
}	
}


#ifndef NDEBUG


//------------------------------------------------------------------------------
/**
\brief  Print debug trace message

The function prints a debug trace message using standard printf

\param[in]      fmt                 Format string
\param[in]      ...                 Arguments to print
*/
//------------------------------------------------------------------------------



//static char buf[64];
void trace(const char* fmt, ...)
{
    va_list argptr;

    va_start(argptr, fmt);
    sprintf(buf, fmt, argptr);
    va_end(argptr);

    print_string(buf);

}
/*
void trace(const char* fmt, ...)
{
    va_list argptr;

    va_start(argptr, fmt);
    vfprintf(stderr, fmt, argptr);
    va_end(argptr);
}
*/
#else
 void trace(const char* fmt, ...) {}
#endif
