/**
********************************************************************************
\file   kernel/edrv.h

\brief  Definitions for Ethernet driver module

This file contains definitions for the Ethernet driver module.
*******************************************************************************/

/*------------------------------------------------------------------------------
Copyright (c) 2016, B&R Industrial Automation GmbH
Copyright (c) 2013, SYSTEC electronic GmbH
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the copyright holders nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
------------------------------------------------------------------------------*/
#ifndef _INC_kernel_edrv_urv_H_
#define _INC_kernel_edrv_urv_H_

#define MAC_BASE	0x84000000
#define BD_NUM		0x40
#define MAC_TX_BD_BASE	(MAC_BASE + 0x400)
#define MAC_RX_BD_BASE	(MAC_TX_BD_BASE + BD_NUM*8)

#define MAC_TX_BD	((Eth_bd_typedef *) MAC_TX_BD_BASE)
#define MAC_RX_BD	((Eth_bd_typedef *) MAC_RX_BD_BASE)

#define ETH_REGS 	((Eth_regs_typedef *) MAC_BASE)
#define PHY_ADDR 	0

#define MII_BUSY	((ETH_REGS->MIISTATUS & 0x02) != 0)

typedef struct 
{
	uint32_t control;
	void * pointer;
} Eth_bd_typedef;

typedef struct 
{

uint32_t	MODER		; 
uint32_t	INT_SOURCE	; 
uint32_t	INT_MASK	; 
uint32_t	IPGT		; 
uint32_t	IPGR1		; 
uint32_t	IPGR2		; 
uint32_t	PACKETLEN	; 
uint32_t	COLLCONF	; 
uint32_t	TX_BD_NUM	; 
uint32_t	CTRLMODER	; 
uint32_t	MIIMODER	; 
uint32_t	MIICOMMAND	; 
uint32_t	MIIADDRESS	; 
uint32_t	MIITX_DATA	; 
uint32_t	MIIRX_DATA	; 
uint32_t	MIISTATUS	; 
uint32_t	MAC_ADDR0	; 
uint32_t	MAC_ADDR1	; 
uint32_t	ETH_HASH0_ADDR	; 
uint32_t	ETH_HASH1_ADDR	; 
uint32_t	ETH_TXCTRL	; 
} Eth_regs_typedef;


#endif /* _INC_kernel_edrv_urv_H_ */
