// mattia
#include <common/oplkinc.h>
#include <kernel/edrv.h>
#include <kernel/dllkfilter.h>
#include <common/target.h>

#include "../drivers/mss_ethernet_mac/mss_ethernet_mac.h"

#define EDRV_MAX_BUFFER_SIZE MSS_MAC_MAX_RX_BUF_SIZE

typedef struct
{
    tEdrvInitParam      initParam;                      // initialization parameters for the EDRV passed from upper layer



} tEdrvInstance;

static void edrv_tx_complete(tEdrvTxBuffer * pBuffer_p);
static void edrv_rx_callback (uint8_t * p_rx_packet,
        uint32_t pckt_length,
        void * p_user_data);

static tEdrvInstance            edrvInstance_l;
 uint8_t rx_buffer1[MSS_MAC_MAX_RX_BUF_SIZE];
 uint8_t rx_buffer2[MSS_MAC_MAX_RX_BUF_SIZE];
 uint8_t rx_buffer3[MSS_MAC_MAX_RX_BUF_SIZE];
//static uint8_t rx_buffer4[MSS_MAC_MAX_RX_BUF_SIZE];

mss_mac_cfg_t mac_config;

static volatile tEdrvRxBuffer           rxBuffer[3];
uint8_t rx_buf0[1516];
uint8_t rx_buf1[1516];
uint8_t rx_buf2[1516];

tOplkError edrv_init(const tEdrvInitParam* pEdrvInitParam_p)
{
    tOplkError  ret = kErrorOk;

    OPLK_MEMSET(&edrvInstance_l, 0, sizeof(edrvInstance_l));

    edrvInstance_l.initParam = *pEdrvInitParam_p;
    DEBUG_LVL_CTRL_TRACE("Initialize edrv...\n");

    target_msleep(500);
    MSS_MAC_cfg_struct_def_init(&mac_config);


    mac_config.phy_addr = MSS_MAC_AUTO_DETECT_PHY_ADDRESS;
    mac_config.speed_duplex_select = MSS_MAC_ANEG_100M_FD;
    mac_config.mac_addr[0] = edrvInstance_l.initParam.aMacAddr[0];
    mac_config.mac_addr[1] = edrvInstance_l.initParam.aMacAddr[1];
    mac_config.mac_addr[2] = edrvInstance_l.initParam.aMacAddr[2];
    mac_config.mac_addr[3] = edrvInstance_l.initParam.aMacAddr[3];
    mac_config.mac_addr[4] = edrvInstance_l.initParam.aMacAddr[4];
    mac_config.mac_addr[5] = edrvInstance_l.initParam.aMacAddr[5];

    MSS_MAC_init(&mac_config);
    MSS_MAC_set_tx_callback(edrv_tx_complete);

    MSS_MAC_receive_pkt(rx_buffer1, (void *)rx_buffer1);
    MSS_MAC_receive_pkt(rx_buffer2, (void *)rx_buffer2);
    MSS_MAC_receive_pkt(rx_buffer3, (void *)rx_buffer3);

    rxBuffer[0].bufferInFrame = 0;
    rxBuffer[1].bufferInFrame = 0;
    rxBuffer[2].bufferInFrame = 0;

    rxBuffer[0].pBuffer = rx_buf0;
    rxBuffer[1].pBuffer = rx_buf1;
    rxBuffer[2].pBuffer = rx_buf2;

    MSS_MAC_set_rx_callback(edrv_rx_callback);
	printf("init eth ok\n");

    target_msleep(500);

    return ret;
}


tOplkError edrv_exit(void)
{
		while(1);
	return kErrorOk;
}


//------------------------------------------------------------------------------
/**
\brief  Get MAC address

This function returns the MAC address of the Ethernet controller

\return The function returns a pointer to the MAC address.

\ingroup module_edrv
*/
//------------------------------------------------------------------------------
const UINT8* edrv_getMacAddr(void)
{
    return edrvInstance_l.initParam.aMacAddr;
}

//------------------------------------------------------------------------------
/**
\brief  Allocate Tx buffer

This function allocates a Tx buffer.

\param[in,out]  pBuffer_p           Tx buffer descriptor

\return The function returns a tOplkError error code.

\ingroup module_edrv
*/
//------------------------------------------------------------------------------

uint16_t malloc_size = 0;

tOplkError edrv_allocTxBuffer(tEdrvTxBuffer* pBuffer_p)
{
    tOplkError          ret = kErrorOk;



    if (pBuffer_p->maxBufferSize > EDRV_MAX_BUFFER_SIZE)
    {
        return kErrorEdrvNoFreeBufEntry;

    }
    uint8_t* txBuffer = malloc( pBuffer_p->maxBufferSize);

    ASSERT(txBuffer);
    if (!txBuffer)

    	return kErrorEdrvNoFreeBufEntry;

    malloc_size +=  pBuffer_p->maxBufferSize;

    pBuffer_p->pBuffer = txBuffer;

    return ret;
}



//------------------------------------------------------------------------------
/**
\brief  Free Tx buffer

This function releases the Tx buffer.

\param[in,out]  pBuffer_p           Tx buffer descriptor

\return The function returns a tOplkError error code.

\ingroup module_edrv
*/
//------------------------------------------------------------------------------
tOplkError edrv_freeTxBuffer(tEdrvTxBuffer* pBuffer_p)
{
    tOplkError          ret = kErrorOk;


    free(pBuffer_p->pBuffer);
    return ret;
}




//------------------------------------------------------------------------------
/**
\brief  Send Tx buffer

This function sends the Tx buffer.

\param[in,out]  pBuffer_p           Tx buffer descriptor

\return The function returns a tOplkError error code.

\ingroup module_edrv
*/
//------------------------------------------------------------------------------
tOplkError edrv_sendTxBuffer(tEdrvTxBuffer* pBuffer_p)
{
    tOplkError          ret = kErrorOk;


    ret = !MSS_MAC_send_pkt(pBuffer_p->pBuffer,     pBuffer_p->txFrameSize, pBuffer_p);
    return ret;
}

static void edrv_tx_complete(tEdrvTxBuffer * pBuffer_p) {

	if(pBuffer_p && pBuffer_p->pfnTxHandler)
		pBuffer_p->pfnTxHandler(pBuffer_p);

}

static UINT8 pkt[64]={0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0x03,0xFF,0x01,0x02,0x03,0xFF,0x01,0x00};
void debug_pkt()
{

	if (!MSS_MAC_send_pkt(pkt, 64, 0))
		printf("Failed to send debug pkt\n");
	else
		printf("Debug pkt sent\n");
}
static volatile void * user_data [2];
static volatile uint32_t newPacket = 0;
static volatile uint32_t int_count = 1234;
//void polling_edrv () {
//	if (newPacket) {
//		int i ; newPacket = 0;
//		if (rxBuffer[0].bufferInFrame == kEdrvBufferLastInFrame && rxBuffer[1].bufferInFrame == 0) i = 0;
//		if (rxBuffer[1].bufferInFrame == kEdrvBufferLastInFrame && rxBuffer[0].bufferInFrame == 0) i = 1;
//		if (rxBuffer[1].bufferInFrame == kEdrvBufferLastInFrame && rxBuffer[0].bufferInFrame == kEdrvBufferLastInFrame) {
//			if (rxBuffer[0].pRxTimeStamp < rxBuffer[1].pRxTimeStamp) { i = 0; } else {i = 1;}
//			 newPacket = 1;
//		}
//
//
//	    int8_t releaseRxBuffer = edrvInstance_l.initParam.pfnRxHandler(&rxBuffer[i]); //pass frame to Powerlink Stack
//
//
//
//	    rxBuffer[i].bufferInFrame = 0; rxBuffer[i].pRxTimeStamp = 0;
//	}
//
//	if (newPacket) polling_edrv();
//}


static void edrv_rx_callback (uint8_t * p_rx_packet,
        uint32_t pckt_length,
        void * p_user_data)
{
  //  tEdrvRxBuffer           rxBuffer;
	int i = 1; int_count++;
	if (p_rx_packet == rx_buffer1) i = 0;

    rxBuffer[i].bufferInFrame = kEdrvBufferLastInFrame;
    rxBuffer[i].rxFrameSize =pckt_length;
    rxBuffer[i].pRxTimeStamp = int_count;

    user_data[i] = p_user_data;
    newPacket = 1;
    memcpy(rxBuffer[i].pBuffer, p_rx_packet, pckt_length);


    int8_t releaseRxBuffer = edrvInstance_l.initParam.pfnRxHandler(&rxBuffer); //pass frame to Powerlink Stack

    if (releaseRxBuffer == kEdrvReleaseRxBufferLater)
    {
    	// stack will call edrv_releaseRxBuffer
    }
    else
      // Packet processing is done, freeing buffer
    	MSS_MAC_receive_pkt((uint8_t *)p_user_data, p_user_data);

}


#if ((CONFIG_DLL_DEFERRED_RXFRAME_RELEASE_SYNC != FALSE) || (CONFIG_DLL_DEFERRED_RXFRAME_RELEASE_ASYNC != FALSE))
//------------------------------------------------------------------------------
/**
\brief  Release Rx buffer

This function releases a late release Rx buffer.

\param[in,out]  pRxBuffer_p         Rx buffer to be released

\return The function returns a tOplkError error code.

\ingroup module_edrv
*/
//------------------------------------------------------------------------------
tOplkError edrv_releaseRxBuffer(tEdrvRxBuffer* pRxBuffer_p)
{
    tOplkError          ret = kErrorOk;

    // Check parameter validity
    ASSERT(pRxBuffer_p != NULL);

   // Freeing the Rx buffer
   MSS_MAC_receive_pkt((uint8_t *)pRxBuffer_p->pBuffer, pRxBuffer_p->pBuffer);


    return ret;
}
#endif

tOplkError edrv_changeRxFilter(tEdrvFilter* pFilter_p,
                               UINT count_p,
                               UINT entryChanged_p,
                               UINT changeFlags_p)
{
	//not used
    return kErrorOk;
}

tOplkError edrv_setRxMulticastMacAddr(const UINT8* pMacAddr_p)
{
	//not used
    return kErrorOk;
}

tOplkError edrv_clearRxMulticastMacAddr(const UINT8* pMacAddr_p)
{
	//not used
    return kErrorOk;
}


