// mattia
#include <common/oplkinc.h>
#include <kernel/edrv.h>
#include <kernel/dllkfilter.h>
#include <common/target.h>
#include <kernel/edrv-urv.h>
#include <kernel/phy.h>
#include <urv/usleep.h>

#define EDRV_MAX_BUFFER_SIZE 1516

#define TX_BD_POINTER		0x85001000
#define RX_BD_POINTER		rx_buffer



typedef struct
{
    tEdrvInitParam      initParam;                      // initialization parameters for the EDRV passed from upper layer



} tEdrvInstance;


static uint8_t __attribute__((section (".dma"))) rx_buffer0[1516];
static uint8_t __attribute__((section (".dma"))) rx_buffer1[1516];


static volatile uint32_t * rx_descriptor0 = (uint32_t *)0x84000000;
static volatile uint32_t * rx_descriptor1 = (uint32_t *)0x84000004;
static volatile uint32_t * tx_descriptor  = (uint32_t *)0x84000008; 
volatile uint32_t * config = (uint32_t *)0x8400000c;

static void edrv_tx_complete(tEdrvTxBuffer * pBuffer_p);
static void edrv_rx_callback (uint8_t * p_rx_packet,
        uint32_t pckt_length,
        void * p_user_data);

static tEdrvInstance            edrvInstance_l;






volatile uint32_t count = 0;



tOplkError edrv_init(const tEdrvInitParam* pEdrvInitParam_p)
{
    tOplkError  ret = kErrorOk;

    OPLK_MEMSET(&edrvInstance_l, 0, sizeof(edrvInstance_l));

    edrvInstance_l.initParam = *pEdrvInitParam_p;
    DEBUG_LVL_EDRV_TRACE("Initialize edrv...\n");




	*rx_descriptor0 = 0x80000000;
	*rx_descriptor1 = 0x80000000;
	*tx_descriptor = 0;
	*config = 3;
unsigned t;
  /* Enable irq: set mie.meie  */
//  asm volatile ("csrrs %0, mie, %1" : "=r"(t) : "r"(1 << 11));
//asm volatile ("nop");asm volatile ("nop");asm volatile ("nop");asm volatile ("nop");
//asm volatile ("csrrsi %0, mstatus, %1" : "=r"(t) : "i"(1 << 3));
	
	rx_buffer0[0] = 0;
    return ret;
}


tOplkError edrv_exit(void)
{
		while(1);
	return kErrorOk;
}


//------------------------------------------------------------------------------
/**
\brief  Get MAC address

This function returns the MAC address of the Ethernet controller

\return The function returns a pointer to the MAC address.

\ingroup module_edrv
*/
//------------------------------------------------------------------------------
const UINT8* edrv_getMacAddr(void)
{
    return edrvInstance_l.initParam.aMacAddr;
}

//------------------------------------------------------------------------------
/**
\brief  Allocate Tx buffer

This function allocates a Tx buffer.

\param[in,out]  pBuffer_p           Tx buffer descriptor

\return The function returns a tOplkError error code.

\ingroup module_edrv
*/
//------------------------------------------------------------------------------


uint16_t malloc_size = 0;


tOplkError edrv_allocTxBuffer(tEdrvTxBuffer* pBuffer_p)
{
    tOplkError          ret = kErrorOk;


    if (pBuffer_p->maxBufferSize > EDRV_MAX_BUFFER_SIZE)
    {
        return kErrorEdrvNoFreeBufEntry;

    }
	
//print_string("buf tx");
    uint8_t* txBuffer = OPLK_MALLOC( pBuffer_p->maxBufferSize);
	//malloc_size += pBuffer_p->maxBufferSize;
    ASSERT(txBuffer);
    if (!txBuffer)

    	return kErrorEdrvNoFreeBufEntry;



    pBuffer_p->pBuffer = txBuffer;
    return ret;
}



//------------------------------------------------------------------------------
/**
\brief  Free Tx buffer

This function releases the Tx buffer.

\param[in,out]  pBuffer_p           Tx buffer descriptor

\return The function returns a tOplkError error code.

\ingroup module_edrv
*/
//------------------------------------------------------------------------------
tOplkError edrv_freeTxBuffer(tEdrvTxBuffer* pBuffer_p)
{
    tOplkError          ret = kErrorOk;

//print_string(" free buf tx");
    OPLK_FREE(pBuffer_p->pBuffer);
    return ret;
}




//------------------------------------------------------------------------------
/**
\brief  Send Tx buffer

This function sends the Tx buffer.

\param[in,out]  pBuffer_p           Tx buffer descriptor

\return The function returns a tOplkError error code.

\ingroup module_edrv
*/
//------------------------------------------------------------------------------
static volatile tEdrvTxBuffer* pBuffer_tx;

tOplkError edrv_sendTxBuffer(tEdrvTxBuffer* pBuffer_p)
{
    tOplkError          ret = kErrorOk;

	while (*tx_descriptor & 0x80000000) ;

	uint32_t tmp = 0x80000000 | (0x0000ffff & (uint32_t)pBuffer_p->pBuffer) | ( (0x0fff0000 & (pBuffer_p->txFrameSize << 16) ));
	pBuffer_tx = pBuffer_p;
	//print_string("TX ");print_word(tmp);print_string("\r\n");
	*tx_descriptor = tmp;
	



    return ret;
}


volatile uint32_t is_isr_running = 0; 
volatile uint32_t frame_count = 0;
void irq_handler ()
{
    tEdrvRxBuffer           rxBuffer;
is_isr_running = 1;
//print_string("!");

if (*tx_descriptor & 0x20000000) {

//print_string("*");

	if(pBuffer_tx && pBuffer_tx->pfnTxHandler)
		pBuffer_tx->pfnTxHandler(pBuffer_tx);

}


    if (!(*rx_descriptor0 & 0x80000000)) {
	
	//print_string("1");
	
  	 rxBuffer.bufferInFrame = kEdrvBufferLastInFrame;
   	 rxBuffer.pBuffer = (UINT8*) rx_buffer0;
   	 rxBuffer.rxFrameSize = 0x0fff & (*rx_descriptor0 >> 16);

	rxBuffer.rxFrameSize = rxBuffer.rxFrameSize-4;
   	 rxBuffer.pRxTimeStamp = frame_count++;

		
   		 edrvInstance_l.initParam.pfnRxHandler(&rxBuffer);
       		  *rx_descriptor0 = 0x80000000 ;
	
	//}

	
     }


    if (!(*rx_descriptor1 & 0x80000000)) {

	
	//print_string("2");
	
  	 rxBuffer.bufferInFrame = kEdrvBufferLastInFrame;
   	 rxBuffer.pBuffer = (UINT8*) rx_buffer1;
   	 rxBuffer.rxFrameSize = 0x0fff & (*rx_descriptor1 >> 16);

	rxBuffer.rxFrameSize = rxBuffer.rxFrameSize-4;
   	 rxBuffer.pRxTimeStamp = frame_count++;

		
   		 edrvInstance_l.initParam.pfnRxHandler(&rxBuffer);
       		  *rx_descriptor1 = 0x80000000 ;
	
	//}

	
	
     }



is_isr_running = 0;
}


#if ((CONFIG_DLL_DEFERRED_RXFRAME_RELEASE_SYNC != FALSE) || (CONFIG_DLL_DEFERRED_RXFRAME_RELEASE_ASYNC != FALSE))
//------------------------------------------------------------------------------
/**
\brief  Release Rx buffer

This function releases a late release Rx buffer.

\param[in,out]  pRxBuffer_p         Rx buffer to be released

\return The function returns a tOplkError error code.

\ingroup module_edrv
*/
//------------------------------------------------------------------------------
tOplkError edrv_releaseRxBuffer(tEdrvRxBuffer* pRxBuffer_p)
{
    tOplkError          ret = kErrorOk;

    // Check parameter validity
    ASSERT(pRxBuffer_p != NULL);

   // Freeing the Rx buffer
   //MSS_MAC_receive_pkt((uint8_t *)pRxBuffer_p->pBuffer, pRxBuffer_p->pBuffer);


    return ret;
}
#endif

tOplkError edrv_changeRxFilter(tEdrvFilter* pFilter_p,
                               UINT count_p,
                               UINT entryChanged_p,
                               UINT changeFlags_p)
{
	//not used
    return kErrorOk;
}

tOplkError edrv_setRxMulticastMacAddr(const UINT8* pMacAddr_p)
{
	//not used
    return kErrorOk;
}

tOplkError edrv_clearRxMulticastMacAddr(const UINT8* pMacAddr_p)
{
	//not used
    return kErrorOk;
}


